@@ -1,7 +1,9 @@
 use crate::impl_json_handling;
-use crate::parameter::{BinaryRecord, ChemicalRecord, Identifier, NoRecord, ParameterError};
-use pyo3::exceptions::PyRuntimeError;
+use crate::parameter::{BinaryRecord, ChemicalRecord, Identifier, ParameterError};
+use either::Either;
+use pyo3::exceptions::{PyRuntimeError, PyValueError};
 use pyo3::prelude::*;
+use std::collections::HashMap;
 
 impl From<ParameterError> for PyErr {
     fn from(e: ParameterError) -> PyErr {
@@ -124,6 +126,8 @@ impl_json_handling!(PyIdentifier);
 ///
 /// Parameters
 /// ----------
+/// identifier : Identifier
+///     The identifier of the pure component.
 /// segments : [str]
 ///     List of segments, that the molecule consists of.
 /// bonds : [[int, int]], optional
@@ -137,34 +141,69 @@ impl_json_handling!(PyIdentifier);
 /// ChemicalRecord
 #[pyclass(name = "ChemicalRecord")]
 #[derive(Clone)]
-#[pyo3(text_signature = "(segments, bonds=None)")]
+#[pyo3(text_signature = "(identifier, segments, bonds=None)")]
 pub struct PyChemicalRecord(pub ChemicalRecord);
 
 #[pymethods]
 impl PyChemicalRecord {
     #[new]
-    fn new(segments: Vec<String>, bonds: Option<Vec<[usize; 2]>>) -> Self {
-        Self(ChemicalRecord::new(segments, bonds))
+    fn new(identifier: PyIdentifier, segments: &PyAny, bonds: Option<&PyAny>) -> PyResult<Self> {
+        if let Ok(segments) = segments.extract::<Vec<String>>() {
+            let bonds = bonds
+                .map(|bonds| bonds.extract::<Vec<[usize; 2]>>())
+                .transpose()?;
+            Ok(Self(ChemicalRecord::new(identifier.0, segments, bonds)))
+        } else if let Ok(segments) = segments.extract::<HashMap<String, f64>>() {
+            let bonds = bonds
+                .map(|bonds| bonds.extract::<HashMap<[String; 2], f64>>())
+                .transpose()?;
+            Ok(Self(ChemicalRecord::new_count(
+                identifier.0,
+                segments,
+                bonds,
+            )))
+        } else {
+            Err(PyValueError::new_err(
+                "`segments` must either be a list or a dict of strings.",
+            ))
+        }
     }
 
     #[getter]
-    fn get_segments(&self) -> Vec<String> {
-        self.0.segments.clone()
-    }
-
-    #[setter]
-    fn set_segments(&mut self, segments: Vec<String>) {
-        self.0.segments = segments
+    fn get_identifier(&self) -> PyIdentifier {
+        PyIdentifier(self.0.identifier().clone())
     }
 
     #[getter]
-    fn get_bonds(&self) -> Vec<[usize; 2]> {
-        self.0.bonds.clone()
+    fn get_segments(&self, py: Python) -> PyObject {
+        match &self.0.segments() {
+            Either::Left(segments) => segments.to_object(py),
+            Either::Right(segments) => segments.to_object(py),
+        }
     }
 
-    #[setter]
-    fn set_bonds(&mut self, bonds: Vec<[usize; 2]>) {
-        self.0.bonds = bonds
+    #[getter]
+    fn get_bonds(&self, py: Python) -> PyObject {
+        match &self.0 {
+            ChemicalRecord::List {
+                identifier: _,
+                segments: _,
+                bonds,
+            } => bonds
+                .iter()
+                .map(|[a, b]| (a, b))
+                .collect::<Vec<_>>()
+                .to_object(py),
+            ChemicalRecord::Count {
+                identifier: _,
+                segments: _,
+                bonds,
+            } => bonds
+                .iter()
+                .map(|([a, b], c)| ((a, b), c))
+                .collect::<HashMap<_, _>>()
+                .to_object(py),
+        }
     }
 }
 
@@ -309,10 +348,6 @@ impl pyo3::class::basic::PyObjectProtocol for PyBinarySegmentRecord {
 
 impl_json_handling!(PyBinarySegmentRecord);
 
-#[pyclass(name = "NoRecord")]
-#[derive(Clone)]
-pub struct PyNoRecord(pub NoRecord);
-
 #[macro_export]
 macro_rules! impl_pure_record {
     ($model_record:ident, $py_model_record:ident, $ideal_gas_record:ident, $py_ideal_gas_record:ident) => {
@@ -324,20 +359,18 @@ macro_rules! impl_pure_record {
         ///     The identifier of the pure component.
         /// molarweight : float
         ///     The molar weight (in g/mol) of the pure component.
-        /// model_record : ModelRecord, optional
+        /// model_record : ModelRecord
         ///     The pure component model parameters.
         /// ideal_gas_record: IdealGasRecord, optional
         ///     The pure component parameters for the ideal gas model.
-        /// chemical_record: ChemicalRecord, optional
-        ///     The chemical record of the pure component.
         ///
         /// Returns
         /// -------
         /// PureRecord
         #[pyclass(name = "PureRecord")]
-        #[pyo3(text_signature = "(identifier, molarweight, model_record=None, ideal_gas_record=None, chemical_record=None)")]
+        #[pyo3(text_signature = "(identifier, molarweight, model_record, ideal_gas_record=None)")]
         #[derive(Clone)]
-        pub struct PyPureRecord(PureRecord<$model_record, $ideal_gas_record>);
+        pub struct PyPureRecord(pub PureRecord<$model_record, $ideal_gas_record>);
 
         #[pymethods]
         impl PyPureRecord {
@@ -345,15 +378,13 @@ macro_rules! impl_pure_record {
             fn new(
                 identifier: PyIdentifier,
                 molarweight: f64,
-                model_record: Option<$py_model_record>,
+                model_record: $py_model_record,
                 ideal_gas_record: Option<$py_ideal_gas_record>,
-                chemical_record: Option<PyChemicalRecord>,
             ) -> PyResult<Self> {
                 Ok(Self(PureRecord::new(
                     identifier.0,
                     molarweight,
-                    chemical_record.map(|cr| cr.0),
-                    model_record.map(|mr| mr.0),
+                    model_record.0,
                     ideal_gas_record.map(|ig| ig.0),
                 )))
             }
@@ -379,13 +410,13 @@ macro_rules! impl_pure_record {
             }
 
             #[getter]
-            fn get_model_record(&self) -> Option<$py_model_record> {
-                self.0.model_record.clone().map($py_model_record)
+            fn get_model_record(&self) -> $py_model_record {
+                $py_model_record(self.0.model_record.clone())
             }
 
             #[setter]
             fn set_model_record(&mut self, model_record: $py_model_record) {
-                self.0.model_record = Some(model_record.0);
+                self.0.model_record = model_record.0;
             }
 
             #[getter]
@@ -397,16 +428,6 @@ macro_rules! impl_pure_record {
             fn set_ideal_gas_record(&mut self, ideal_gas_record: $py_ideal_gas_record) {
                 self.0.ideal_gas_record = Some(ideal_gas_record.0);
             }
-
-            #[getter]
-            fn get_chemical_record(&self) -> Option<PyChemicalRecord> {
-                self.0.chemical_record.clone().map(PyChemicalRecord)
-            }
-
-            #[setter]
-            fn set_chemical_record(&mut self, chemical_record: PyChemicalRecord) {
-                self.0.chemical_record = Some(chemical_record.0);
-            }
         }
 
         #[pyproto]
@@ -645,7 +666,7 @@ macro_rules! impl_parameter_from_segments {
             ///
             /// Parameters
             /// ----------
-            /// pure_records : [PureRecord]
+            /// chemical_records : [ChemicalRecord]
             ///     A list of pure component parameters.
             /// segment_records : [SegmentRecord]
             ///     A list of records containing the parameters of
@@ -653,14 +674,14 @@ macro_rules! impl_parameter_from_segments {
             /// binary_segment_records : [BinarySegmentRecord], optional
             ///     A list of binary segment-segment parameters.
             #[staticmethod]
-            #[pyo3(text_signature = "(pure_records, segment_records, binary_segment_records=None)")]
+            #[pyo3(text_signature = "(chemical_records, segment_records, binary_segment_records=None)")]
             fn from_segments(
-                pure_records: Vec<PyPureRecord>,
+                chemical_records: Vec<PyChemicalRecord>,
                 segment_records: Vec<PySegmentRecord>,
                 binary_segment_records: Option<Vec<PyBinarySegmentRecord>>,
             ) -> Result<Self, ParameterError> {
                 Ok(Self(Rc::new(<$parameter>::from_segments(
-                    pure_records.into_iter().map(|pr| pr.0).collect(),
+                    chemical_records.into_iter().map(|cr| cr.0).collect(),
                     segment_records.into_iter().map(|sr| sr.0).collect(),
                     binary_segment_records.map(|r| r.into_iter().map(|r| BinaryRecord{id1:r.0.id1,id2:r.0.id2,model_record:r.0.model_record.into()}).collect()),
                 )?)))
